/* jshint esversion : 6 */

(function appIo() {
    const module = angular.module('appIo', []);



    class Network {
        constructor(o) {
            if (!o) o = {};

            const { protocol = "http", address = "127.0.0.1", port = 80, path = "" } = o;
            this.protocol = protocol;
            this.ipAddress = address;
            this.port = port;
            this.path = path;
            this.address = (address + ':' + port);
            this.url = (protocol + '://' + this.address + '/' + path);
        }
    }

    const AppSocket = ($rootScope, $timeout) => {
        return o => {

            let log = true;
            let ready = $q.defer();
            let connectionAttemptNumber = 0;
            let online = true;
            const _setReady = (v, e) => {
                if (e) ready.notify(e);
                ready[v ? 'resolve' : 'reject']();
                $timeout(() => ready = $q.defer(), 600);
            };
            const _broadcastOnlineEvent = b => $rootScope.$emit('socket::connectionEvent', b);

            function SocketEvent(ev) {
                return fn => {
                    this.socket.on(ev, e => {

                        if ("connect" === ev)
                            connectionAttemptNumber = 0;
                        else if (["reconnect", "reconnect_attempt", "reconnecting"].indexOf(ev) > -1)
                            connectionAttemptNumber = e;

                        online = ("connection" === ev || "connect" === ev || "reconnect" === ev);

                        _setReady(e, online);
                        _broadcastOnlineEvent({
                            evName: ev,
                            evDes: e,
                            connected: online
                        });


                        if (fn) fn.call(this, e);

                        if (log) console.log('io::' + ev, e);
                    });
                };
            }
            return (new class AppSocket extends Network {
                constructor({ network, namespace }) {
                    super(network);



                    this.socket = (function(namespace) {
                        let ns = null;
                        if (namespace)
                            ns = (namespace.startsWith('/') ? namespace : ('/' + namespace));

                        return io(ns);
                    }(namespace));

                    const socket_events = [
                        "connect",
                        "connect_error",
                        "connect_timeout",
                        "error",
                        "disconnect",
                        "reconnect",
                        "reconnect_attempt",
                        "reconnect_error",
                        "reconnect_failed",
                    ];
                    socket_events.forEach(v => this["on_" + v] = SocketEvent.call(this, v));

                }

                get connectionAttemptNumber() {
                    return connectionAttemptNumber;
                }
                get online() {
                    return online;
                }

                emit() {
                    const a = Array.from(arguments);
                    this.socket.emit.apply(this.socket, a);
                }
                on(evName, fn) {
                    this.socket.on(evName, res => fn.call(this, res));
                }
                once(evName, fn) {
                    this.socket.once(evName, res => fn.call(this, res));
                }
                hasListeners(evName) {
                    return this.socket.hasListeners(evName);
                }
                removeListener(evName) {
                    this.socket.removeListener(evName);
                }

                ping(fn) {
                    if (fn) {
                        this.socket.on('ping', () => {});
                        this.socket.on('pong', latency => fn.call(this, latency));
                    }
                }
                set enableLog(l) {
                    log = l ? true : false;
                }
                get enableLog() {
                    return log;
                }

            }(o));
        };
    };
    AppSocket.$inject = ['$rootScope', '$timeout'];
    module.factory('appSocket', AppSocket);


    const app_sockets = { cfg: {}, socket: {} };
    const AppIO = {
        addConnection(name, opt) {
            if (!app_sockets.cfg[name])
                app_sockets.cfg[name] = opt || null;

            return this;
        },
        $get(appSocket) {
            return n => {
                if (!app_sockets.socket[n])
                    app_sockets.socket[n] = appSocket(app_sockets.cfg[n]);

                return app_sockets.socket[n];
            };
        }
    };
    module.provider('appIo', AppIO);

}());