/* jshint esversion : 6 */

(function appManifest() {

    const module = angular.module('AppManifest', []);


    function AppManifest() {

        let isUpdated = false;

        const cache = window.applicationCache;
        const ready = $q.defer();

        const o = Object.create({});

        Object.defineProperty(o, 'status', {
            get: () => {
                const status = cache.status;

                if (cache.UNCACHED == status) // UNCACHED == 0
                    return 'UNCACHED';
                else if (cache.IDLE == status) // IDLE == 1
                    return 'IDLE';
                else if (cache.CHECKING == status) // CHECKING == 2
                    return 'CHECKING';
                else if (cache.DOWNLOADING == status) // DOWNLOADING == 3
                    return 'DOWNLOADING';
                else if (cache.UPDATEREADY == status) // UPDATEREADY == 4
                    return 'UPDATEREADY';
                else if (cache.OBSOLETE == status) // OBSOLETE == 5
                    return 'OBSOLETE';
                else
                    return 'UKNOWN CACHE STATUS';
            }
        });

        const evs = ['cached', 'checking', 'downloading', 'error', 'noupdate', 'obsolete', 'progress', 'updateready'];
        evs.forEach(ev => {
            const onEv = 'on' + ev;
            Object.defineProperty(o, onEv, {
                value: fn => {
                    if (fn) {
                        cache[onEv] = () => fn.call(this);
                        cache.addEventListener(ev, e => fn.call(this, e), false);
                    }
                }
            });
        });

        Object.defineProperty(o, 'isUpdated', { get: () => isUpdated });

        Object.defineProperty(o, 'run', {
            value: function() {

                const setReady = () => {
                    ready.notify(cache);



                    if (!isUpdated)
                        isUpdated = (cache.status === 4 || cache.status === 3);


                    if ([1, 4].indexOf(cache.status) > -1)
                        ready.resolve(this.status);
                };

                const setError = e => {
                    console.warn(e);
                    ready.reject(e);
                };

                evs.forEach(ev => {
                    const onEv = 'on' + ev;
                    this[onEv](e => ev == 'error' ? setError(e) : setReady());
                });

                try {
                    cache.update(); //Attempt to update the user's cache.
                } catch (e) {
                    ready.notify(e);
                }


            }
        });

        Object.defineProperty(o, 'ready', { get: () => ready.promise });

        o.run();

        return o;
    }
    module.factory('appManifest', AppManifest);

}());