/* jshint esversion : 6 */

const angular_module = require('./module');

angular_module.directive('isIntoView', ($timeout, $document) => {
    return {

        scope: {
            isIntoView: '&'
        },

        link: (scope, element, attrs) => {

            const o = {
                get pageTop() {
                    return window.scrollY;
                },

                get pageBottom() {
                    const pageTop = this.pageTop;
                    return pageTop + (Math.max(document.documentElement.clientHeight, window.innerHeight || 0));
                },

                get elementTop() {
                    const el = element[0];
                    return (el.getBoundingClientRect().top + document.body.scrollTop);
                },

                get elementBottom() {
                    const el = element[0];
                    return (this.elementTop + el.offsetHeight);
                },

                isFullyIntoView(pageTop, pageBottom, elementTop, elementBottom) {
                    if (element)
                        return ((pageTop < elementTop) && (pageBottom > elementBottom));
                    else
                        return false;
                },

                isPartiallyIntoView(pageTop, pageBottom, elementTop, elementBottom) {
                    if (element)
                        return ((elementTop <= pageBottom) && (elementBottom >= pageTop));
                    else
                        return false;
                },

                get isIntoView() {
                    if (element) {
                        const pageTop = this.pageTop;
                        const pageBottom = this.pageBottom;

                        const elementTop = this.elementTop;
                        const elementBottom = this.elementBottom;

                        if (attrs.fullyInView)
                            return this.isFullyIntoView(pageTop, pageBottom, elementTop, elementBottom);
                        else
                            return this.isPartiallyIntoView(pageTop, pageBottom, elementTop, elementBottom);

                    } else
                        return false;

                }
            };



            const scroller = angular.element(document.getElementById('view-scroller'));

            const exec = () => {
                if (o.isIntoView && scope.isIntoView)
                    $timeout(scope.isIntoView, 0);
            };

            scroller.on('scroll', () => exec());

            let executed = 0;

            if (!scope.$eval(attrs.disableIntoViewOnInit)) {
                const delay = scope.$eval(attrs.delay);

                const a = $timeout(() => {
                    let x = scope.$watch(() => o.isIntoView, n => {
                        if (!scope.$eval(attrs.enableRepeat)) {
                            exec();
                            x();
                            $timeout.cancel(a);
                            x = null;
                        }
                    });
                }, delay || 0, true);
            }

        }
    };
});