/* jshint esversion : 6 */

const angular_module = require('./module');
const DynamicItems = require('./dynamicItems');

const isObject = x => !Array.isArray(x) && typeof x === 'object';

const DeferredLoadingView = function() {

    return function(data, deferredFn, pageSize) {

        const keys = isObject(data) ? Object.keys(data) : null;

        class DynamicDeferred extends DynamicItems {
            constructor(pageSize) {
                super(pageSize);
            }

            fetchPage_(pageNumber) {
                this.loadedPages[pageNumber] = null;

                (isPromise(deferredFn) ? deferredFn : deferredFn()).then(() => {
                    this.loadedPages[pageNumber] = [];
                    const pageOffset = pageNumber * this.PAGE_SIZE;

                    for (let i = pageOffset; i < pageOffset + this.PAGE_SIZE; i++) {
                        const k = keys ? keys[i] : i;
                        this.loadedPages[pageNumber].push(data[k]);
                    }

                });
            }

            fetchNumItems_() {
                if (Array.isArray(data))
                    this.numItems = data.length;
                else if ("object" === typeof data)
                    this.numItems = Object.keys(data).length;
            }
        }

        return new DynamicDeferred(pageSize);
    };
};


angular_module.factory('deferredLoadingView', DeferredLoadingView);