/* jshint esversion : 6 */

const genId = function() {
    let text = "";
    let possible = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";

    for (var i = 0; i < 10; i++)
        text += possible.charAt(Math.floor(Math.random() * possible.length));

    return text;
};

function FormField(name, opt) {
    if (!opt) opt = {};

    const form_field = {};
    form_field.name = name;
    const {
        value = "", type = "text", required = false, label = null, icon = null, group, flex, flexOffset, autocomplete = false, id = genId()
    } = opt;
    form_field.value = value;
    form_field.type = type;
    form_field.required = required;
    form_field.error = {};
    form_field.label = label;
    form_field.icon = icon;
    form_field.group = group;
    form_field.flex = flex;
    form_field.flex_offset = flexOffset;
    form_field.autocomplete = autocomplete;
    form_field.id = id;
    const mq = [
        '-xs', '-gt-xs',
        '-sm', '-gt-sm',
        '-md', '-gt-md',
        '-lg', '-gt-lg', '-xl'
    ];
    mq.map(e => 'flex' + e).forEach(v => form_field[v] = opt[v.replace(/\-/g, '_')] || flex);
    mq.map(e => 'flex_offset' + e).forEach(v => form_field[v] = opt[v.replace(/\-/g, '_')] || flexOffset);
    form_field.validation = (function({ validation, type }) {
        const r = {};

        if (!validation || (type && type === 'number'))
            return "none";
        else if ("string" === typeof validation || RegExp === validation.constructor)
            r.pattern = validation;

        Object.entries(validation).forEach(e => {
            const [k, v] = e;
            r[k] = v;
        });

        return r;
    }(opt));
    form_field.regExp = (function(v) {
        if (v !== 'none')
            return ("string" === typeof v.pattern ? (v.flags ? new RegExp(v.pattern, v.flags) : new RegExp(v.pattern)) : v.pattern);
        else
            return null;
    }(form_field.validation));
    Object.defineProperty(form_field, 'validPattern', {
        get: function() {
            this.error.pattern = false;

            if (this.validation === 'none')
                return true;
            else if (this.value != null && "undefined" !== typeof this.value && this.value.toString().length) {
                this.error.pattern = !this.regExp.test(this.value);
                return !this.error.pattern;
            } else
                return true;
        }
    });
    Object.defineProperty(form_field, 'requiredEditing', {
        get: function() {
            if (this.required && this.value != null && "undefined" !== typeof this.value) {
                this.error.required = this.value.toString().length === 0;
            } else
                this.error.required = false;

            return !this.error.required;
        }
    });
    Object.defineProperty(form_field, 'valid', {
        get: function() {
            return (this.validPattern && this.requiredEditing);
        }
    });
    Object.defineProperty(form_field, '$error', {
        get: function() {
            const a = {};

            const valid = this.valid;

            if (this.value != null && 'undefined' !== typeof this.value && this.value.toString().length)
                this.class = valid ? 'valid' : 'not-valid';

            for (let k in this.error) {
                if (this.error[k]) {
                    a[k] = !valid;
                    if (!valid)
                        this.error[k] = false;
                } else
                    continue;
            }

            return a;
        }
    });


    return form_field;
}

module.exports = FormField;