/* jshint esversion : 6*/

const angular_module = require('./module');
const Form = require('./Form');

const forms = {};
const FormProvider = {
    addForm(name, opt, fn) {

        if (!forms.hasOwnProperty(name))
            forms[name] = Form(name, opt);

        if (fn)
            fn.call(this, forms[name]);

        return this;
    },
    $get($http, $timeout, $location) {
        return name => {


            let flashMessage = null,
                flashMessageCod = null;

            const m = class {
                constructor(name) {
                    angular.extend(this, forms[name]);
                }

                get flashMessage() {
                    return flashMessage;
                }

                get groups() {
                    return Object.values(this._groups).map(e => e.fields)
                }


                sendHttpRequest(fn) {
                    const { request: req, fields } = this;

                    req.data = {};
                    angular.forEach(fields, ({ name, value }, k) => req.data[name] = value);
                    req.transformResponse = (data, headersGetter, status) => {
                        let r;
                        try {
                            r = JSON.parse(data);
                        } catch (e) {
                            r = data;
                        }
                        return r;
                    };

                    if (!fn) {

                        const __locationAssign = (path) => {
                            const a = [
                                ($location.protocol() + '://'), ($location.host() + ':'), $location.port(), (path.match(/^\//) ? path : ('/' + path))
                            ].join("");

                            window.location.assign(a);
                        };


                        const exec = ({ data, status }) => {
                            const { data: res, error, locationRedirect, urlRedirect, hrefRedirect } = data;

                            flashMessage = error ? error : (typeof res === "string" ? res : null);

                            const { failureRedirect, successRedirect } = this;

                            $timeout(() => {
                                if (hrefRedirect) {
                                    window.location.href = hrefRedirect;
                                }
                                if (urlRedirect) {
                                    __locationAssign(urlRedirect);
                                } else if (locationRedirect || failureRedirect || successRedirect) {
                                    if (locationRedirect)
                                        $location.path(locationRedirect);
                                    else
                                        $location.path(error ? failureRedirect : successRedirect);
                                }
                            }, 800, true);

                        };
                        $http(req).then(exec, exec).catch(angular.noop);
                    } else if (typeof fn === "function")
                        $http(req).then(fn, fn).catch(angular.noop);

                }


                get requirementsCompleted() {
                    let r = 1;

                    for (let i = 0; i < this.fields.length; i++) {

                        const { value: v, required } = this.fields[i];

                        if (!required || v !== null && typeof v !== "undefined" && v.toString().length)
                            continue;
                        else {
                            r = 0;
                            break;
                        }
                    }

                    return r;
                }


                get fullyValidated() {
                    let r = 1;
                    for (let i = 0; i < this.fields.length; i++) {
                        if (this.fields[i].valid)
                            continue;
                        else {
                            r = 0;
                            break;
                        }
                    }
                    return r;
                }


                get valid() {
                    return (this.requirementsCompleted && this.fullyValidated);
                }

            };
            return new m(name);
        };
    }
};
angular_module.provider('appForm', FormProvider);