/* jshint esversion : 6 */

const angular_module = require('./module');

angular_module.directive('appFormAuthInput', () => {
    return {
        restrict: 'A',
        require: ['^^appFormAuthentication', 'ngModel']
    };
});

angular_module.directive('appFormBtn', () => {
    return {
        restrict: 'A',
        require: '^^appFormAuthentication',
        template: '<ng-transclude></ng-transclude>',
        transclude: true,
        scope: {
            sendHttpRequest: '&',
            getFields: '&',
            getFieldsValue: '&'
        },
        link: ($scope, elem, attrs, ctrl) => {

            const { sendHttpRequest, getFields, getFieldsValue } = $scope;

            if (sendHttpRequest) {
                elem.on('click', () => ctrl.sendHttpRequest(sendHttpRequest));
            } else if (getFields) {
                elem.on('click', () => getFields(ctrl.fields));
            } else if (getFieldsValue) {
                elem.on('click', () => getFieldsValue(Object.values(ctrl.fields).map(e => e.value)));
            }
        }
    };
});

angular_module.directive('appFormAuthentication', $timeout => {
    return {
        restrict: 'E',
        controller: ($scope, appForm) => appForm($scope.name),
        controllerAs: '$form',
        scope: {
            name: '@',
            btnTxt: '@?',
            action: '@?',
            method: '@?',
        },
        template: template(),
        transclude: {
            appFormBtn: '?appFormBtn'
        },
        link: (scope, elem, attrs, ctrl) => {

            const form = elem.find('form');

            angular.element(window).on('keyup', ev => {
                if (ev.keyCode === 13 && ctrl.fullyValidated)
                    ctrl.sendHttpRequest();
            });

            if (ctrl.bindToForm) {
                $timeout(() => {
                    form.attr('METHOD', ctrl.method);
                    form.attr('ACTION', ctrl.action);
                }, 0, true);
            }

        }
    };
});


function template() {

    const mq = [
        '-xs', '-gt-xs',
        '-sm', '-gt-sm',
        '-md', '-gt-md',
        '-lg', '-gt-lg', '-xl'
    ];

    let mdInputContainer = '<md-input-container ng-repeat="(k, field) in groups"  flex="{{field.flex}}" flex-offset="{{field.flex_offset}}" ';

    mq.map(e => 'flex' + e).forEach(v => {
        mdInputContainer += v + '="{{field[\'' + v + '\']}}" ';
    });
    mq.map(e => 'flex_offset' + e).forEach(v => {
        mdInputContainer += v + '="{{field[\'' + v + '\']}}" ';
    });

    mdInputContainer += '>';


    return '' +
        '<div layout="row" flex="grow" layout-align="center center">' +
        '   <md-content layout-padding flex>' +
        '       <h1 ng-if="$form.title">{{$form.title}}</h1>' +
        '       <form name="name" layout="column" flex>' +
        '           <div ng-repeat="groups in $form.groups" flex="none" layout="row">' + mdInputContainer +
        '                   <label ng-class="field.class === \'valid\' ? $form.validIconClass : field.class === \'not-valid\' ? $form.errorIconClass : \'\'">{{field.label || field.name}}</label>' +
        '                   <md-icon ng-if="field.icon" ng-bind="field.icon" ng-class="field.class === \'valid\' ? $form.validIconClass : field.class === \'not-valid\' ? $form.errorIconClass : \'\'"></md-icon>' +
        '                   <input app-form-auth-input ng-class="field.class" autocomplete="{{field.autocomplete}}" id="{{field.id}}" type="{{field.type}}" name="{{field.name}}" ng-required="{{field.required}}" ng-model="field.value">' + '</ng-repeat>' +
        '                   <div ng-messages="field.$error" ng-hide="false">' +
        '                       <div ng-message="pattern">{{field.validation.message || \'not valid\'}}</div>' +
        '                   </div>' +
        '               </md-input-container>' +
        '           </div>' +
        '           <div layout="row" flex="none" layout-padding>' +
        '               <div flex="80" ng-if="$form.enableFlashMessage" layout="row">' +
        '                   <div flex layout="row" layout-align="start center" ng-show="$form.flashMessage">' +
        '                       <md-icon ng-bind="\'sentiment_neutral\'"></md-icon>' +
        '                       <div flex="80" layout="row">' +
        '                           <span>{{$form.flashMessage}}</span>' +
        '                       </div>' +
        '                   </div>' +
        '               </div>' +
        '               <div flex="{{$form.enableFlashMessage ? \'20\' : \'auto\'}}" layout="row" layout-align="end center" ng-transclude="appFormBtn">' +
        '                   <md-button ng-if="!$form.bindToForm" aria-label="{{field.name}}" class="md-fab" ng-click="$form.sendHttpRequest()" ng-disabled="!$form.valid" ng-class="!$form.valid ? $form.disableClass : $form.enableClass">' +
        '                       <md-icon ng-bind="\'send\'" ng-class="!$form.valid ? $form.disableIconClass : $form.enableIconClass"></md-icon>' +
        '                   </md-button>' +
        '                   <button type="submit" ng-if="$form.bindToForm" class="md-button md-icon-button" ng-disabled="!$form.valid" ng-class="!$form.valid ? $form.disableClass : $form.enableClass">' +
        '                       <md-icon ng-bind="\'send\'"  ng-class="!$form.valid ? $form.disableIconClass : $form.enableIconClass"></md-icon>' +
        '                   </button>' +
        '               </div>' +
        '           </div>' +
        '       </form>' +
        '   </md-content>' +
        '</div>';
}