/* jshint esversion : 6 */

const angular_module = require('./module');
const array_chunk = require('./array_chunk');
//const Fuse = require('fuse-js-latest');


class SearchInterval {
    constructor() {
        this.interval = 0;
        this._fn = null;
    }

    set fn(fn) {
        this._fn = fn;
    }

    get fn() {
        return this._fn;
    }

    clearInterval() {
        const q = $q.defer();
        if (this.fn) {
            try {
                this.interval = 0;
                clearInterval(this._fn);
                delete this._fn;
                q.resolve();
            } catch (e) {
                q.reject(e);
            }
        } else
            q.resolve();

        return q.promise;
    }

    setInterval(txt) {
        const q = $q.defer();

        if (!this.fn) {
            this.fn = setInterval(() => {

                this.interval++;
                const y = this.interval % (txt.split(/\s+/g).length + 1);

                if (!y) {
                    this.clearInterval();
                    q.resolve(txt);
                }
            }, 800);
        } else
            q.reject();

        return q.promise;
    }
}

class SearchWorker extends SearchInterval {
    constructor() {
        super();
        this.worker = null;
    }

    setSearchInterval(txt) {
        return this.setInterval(txt).then(txt => this.setWorker(txt));
    }

    terminateWorker() {
        if (this.worker) {
            this.worker.terminate();
            this.worker = null;
        }
        this.clearInterval();
    }

    setWorker(txt) {
        const q = $q.defer();

        if (!txt || !txt.length) {
            this.terminateWorker();
            q.reject();
        } else {
            if (!this.worker) {
                this.worker = new Worker('fuse/fuse.js');
                q.resolve(txt);
            } else
                q.resolve(txt);
        }

        return q.promise;
    }

    postMessage(a) {
        if (this.worker)
            this.worker.postMessage(a);
    }

    set onmessage(fn) {
        if (this.worker)
            this.worker.onmessage = fn;
    }

}


const FullTextSearch = (data, options) => {


    const __searchWorker = new SearchWorker();

    return (txt, ev) => {
        const q = $q.defer();
        __searchWorker.terminateWorker();

        if (!txt || !txt.length)
            q.resolve(data);

        __searchWorker.setWorker(txt)
            .then(
                txt => {

                    __searchWorker.postMessage({ data, options, txt });

                    __searchWorker.onmessage = ({ data: r }) => {
                        __searchWorker.terminateWorker();

                        if (!r || !r.length)
                            q.reject();
                        else
                            q.resolve(r);
                    };
                },
                () => {
                    q.reject();
                    __searchWorker.terminateWorker();
                }
            )
            .catch(() => {
                q.reject();
                __searchWorker.terminateWorker();
            })
            .done();



        return q.promise;
    };
};



const ChunkSearch = function() {
    return function(data, options) {
        if (!options) options = {};


        const __chunk = array_chunk(data, (options.chunkSize || Math.floor(data.length / 10)));
        const queue = __chunk.map(e => FullTextSearch(e, options));

        return (txt, ev) => {
            const q = $q.defer();

            const z = queue.map(e => e(txt, ev));
            const init = z.shift();

            const x = [];

            $q.allSettled(queue.map(e => e(txt, ev)))
                .then(
                    r => {

                        const x = r.reduce((curr, next) => {
                            if (next.state === 'fulfilled')
                                return curr.concat(next.value);

                            return curr;
                        }, []);



                        if (!x.length)
                            q.reject();
                        else {
                            const a = x[0];
                            const isObject = x => !Array.isArray(x) && typeof x === 'object';

                            if (a != null && isObject(a) && a.hasOwnProperty('item'))
                                q.resolve(x.map(e => e.item));
                            else
                                q.resolve(x);
                        }

                    }, q.reject
                )
                .catch(q.reject)
                .done();



            return q.promise;
        };




    };
};


angular_module.factory('fullTextSearch', ChunkSearch);