/* jshint esversion : 6 */

const { gulp, concat, babel, babelify, browserify, buffer, source, uglify, sort, wrap, strip, del } = require('./gulp_plugins');


const assets = require('./assets/gulpfile').assets;
const dynamicView = require('./dynamicView/gulpfile').dynamic_view;
const formProvider = require('./formProvider/gulpfile').form_provider;
const fullTextSearch = require('./fullTextSearch/gulpfile').full_text_search;
const idb = require('./indexed_db/gulpfile').idb;

const run_build_task = gulp.parallel(assets, dynamicView, formProvider, fullTextSearch, idb);



const __build = () => {
    return gulp.src(`${__dirname}/*/index.js`)
        .pipe(concat('index.js'))
        .pipe(strip())
        .pipe(wrap({ src: 'wrap.txt' }))
        .pipe(gulp.dest(__dirname));
};


const __minify = () => {
    const src = `${__dirname}/index.js`;
    const bundler = browserify(src);
    bundler.transform(babelify);

    return bundler.bundle()
        .on('error', function(err) { console.error(err); })
        .pipe(source('index.js'))
        .pipe(buffer())
        .pipe(strip())
        .pipe(gulp.dest(__dirname));


};


gulp.task('default', gulp.series(run_build_task, __build, done => done()));

/* 


const __build = () => {
    const src = `${__dirname}/src/index.js`;
    const bundler = browserify(src);
    //bundler.transform(babelify, { presets: ["@babel/preset-env"] });

    return bundler.bundle()
        .on('error', function(err) { console.error(err); })
        .pipe(source(`${__dirname}/index.js`))
        .pipe(buffer())
        .pipe(strip())
        .pipe(gulp.dest(__dirname));
};


exports.full_text_search = __build;
gulp.task('full_text_search', __build); */