/* jshint esversion : 6 */

const angular_module = require('./module');
const IDB = function($http, IDBStore_setter, $idb) {
    return function(dbName) {

        class Injection {
            constructor($http, IDBStore_setter, $idb) {
                this.$http = $http;
                this.IDBStore_setter = IDBStore_setter;
                this.$idb = $idb;
            }
        }

        class StoresSetter extends Injection {
            constructor($http, IDBStore_setter, $idb) {
                super($http, IDBStore_setter, $idb);
                this.__stores = {};
            }
            _getStore(storeName) {
                return this.__stores[storeName];
            }

            _existStore(storeName) {
                return this.__stores.hasOwnProperty(storeName);
            }

            _addStore(db, dbName, storeName) {
                this.__stores[storeName] = this.IDBStore_setter(db, dbName, storeName);
            }

            _initStore(db, dbName, storeName, httpStatus) {
                if (!this._existStore(storeName))
                    this._addStore(db, dbName, storeName);

                return this._getStore(storeName).onInit(httpStatus);
            }
        }


        const dbReady = $q.defer(),
            ready = $q.defer();

        class IDB extends StoresSetter {
            constructor($http, IDBStore_setter, $idb, dbName) {
                super($http, IDBStore_setter, $idb);
                this.dbName = dbName.capitalize();
            }

            onInit() {
                this.dbReady
                    .then(
                        () => {

                            console.log(this.objectStores)

                            const x = Object.values(this.objectStores)[0];

                            const init = x.shift();

                            x.reduce(
                                    (curr, next, index) => {
                                        return curr
                                            .then(
                                                () => this._setStore(next.name),
                                                ready.reject,
                                                ready.notify
                                            )
                                            .catch(ready.reject);
                                    }, this._setStore(init.name)
                                )
                                .then(ready.resolve, ready.reject, ready.notify)
                                .catch(ready.reject)
                                .done();

                        },
                        ready.reject,
                        ready.notify
                    )
                    .catch(ready.reject)
                    .done();

                return ready.promise;
            }

            get dbReady() {
                return dbReady.promise;
            }

            get ready() {
                return ready.promise;
            }

            _getDBconfig(callback) {
                return this.$http({
                        method: 'POST',
                        url: '/' + this.dbName.toLowerCase() + '/'
                    })
                    .then(callback, callback)
                    .catch(e => angular.noop);
            }

            _setDB() {

                this._getDBconfig(httpRes => {

                    const __dbName = this.dbName.capitalize();
                    const __oldConfig = $localStorage.getItem(__dbName);

                    if (!httpRes || !httpRes.data) httpRes = {};
                    const { status = -1, data = __oldConfig } = httpRes;
                    this.httpStatus = status;

                    this.dbName = data.dbName;

                    this.objectStores = angular.copy(data);

                    if (!__oldConfig) {
                        $localStorage.setItem(__dbName, data);
                    }

                    delete this.objectStores.dbName;

                    ready.notify({ value: { dbName: this.dbName, ev: 'db config' } });

                    if (status === 200 || __oldConfig) {
                        $appIDB.setDb(this.dbName, this.objectStores)
                            .then(
                                () => {
                                    this.db = this.$idb.useDb(this.dbName);
                                    dbReady.resolve();
                                },
                                dbReady.reject,
                                dbReady.notify
                            )
                            .catch(dbReady.reject)
                            .done();

                    } else
                        dbReady.reject();
                });

                return this.dbReady;
            }

            _setStore(storeName) {
                return this._initStore(this.db, this.dbName, storeName, this.httpStatus);
            }

            get allHttpDataLoaded() {
                const httpReady = $q.defer();

                $q.when(this.dbReady,
                        () => {
                            const _stores = Object.values(this.__stores);
                            const queue = _stores.map(e => e.httpRequest);

                            $q.all(queue)
                                .then(httpReady.resolve, httpReady.reject, httpReady.notify)
                                .catch(httpReady.resolve)
                                .done();

                        },
                        httpReady.reject,
                        httpReady.notify
                    )
                    .catch(httpReady.reject)
                    .done();

                return httpReady.promise;
            }


        }

        return new IDB($http, IDBStore_setter, $idb, dbName);
    };
};
IDB.$inject = ['$http', 'IDBStore_setter', '$idb'];
angular_module.service('IDB_setter', IDB);