/* jshint esversion : 6 */


const angular_module = require('./module');
const IDBStore = function($http, toDay) {
    return function(db, dbName, storeName) {

        class Injection {
            constructor($http, toDay) {
                this.$http = $http;
                this.toDay = toDay;
            }
        }


        const defer = $q.defer();
        class DeferEvents extends Injection {
            constructor($http, toDay) {
                super($http, toDay);
            }
            _notify({ getHttpData, ev, index, dataLength }) {
                const n = {
                    dbName: this.dbName,
                    storeName: this.storeName,
                    getHttpData: getHttpData,
                    ev: ev
                };
                if (index)
                    n.index = index;
                if (dataLength)
                    n.length = dataLength;

                defer.notify(n);
            }

            _resolve(getHttpData) {
                this._notify({
                    ev: getHttpData ? "update ok" : "offline",
                    getHttpData: getHttpData
                });
                return defer.resolve();
            }

            get ready() {
                return defer.promise;
            }
        }


        const httpDefer = $q.defer();
        class IDBStore extends DeferEvents {
            constructor($http, toDay, db, dbName, storeName) {
                super($http, toDay);
                this.objectStore = db.objectStore(storeName);
                this.dbName = dbName.capitalize();
                this.storeName = storeName;
                this.url = '/' + dbName.toLowerCase() + '/' + storeName + '/';
                this._data = [];
                this.offlineDataAvaiable = false;
            }

            onInit(status) {


                const _getStoreData = !this._localParams || this._localParams && this._localParams.lastDataUpdate && this._localParams.lastDataUpdate.date !== this.toDay();


                try {
                    this._getAll()
                        .then(
                            localData => {
                                if (200 !== status || !_getStoreData) {
                                    this._done({ data: localData, getHttpData: 0, lastUpdate: 0 });
                                } else {
                                    this._getStoreData(localData)
                                        .then(
                                            httpRes => {
                                                if (!httpRes) httpRes = {};
                                                const { status = -1, data: remoteData } = httpRes;

                                                if (status !== 200 || !remoteData || !remoteData.length) {
                                                    this._done({ data: localData, getHttpData: 0, lastUpdate: status === 200 });
                                                } else {
                                                    this._updateLocalData({ remoteData: remoteData, localData: localData });
                                                }
                                            },
                                            () => {
                                                this._done({ data: localData, getHttpData: 1, lastUpdate: 0 });
                                            }
                                        );
                                }
                            },
                            err => defer.reject(err)
                        )
                        .catch(err => defer.reject(err))
                        .done();
                } catch (e) {
                    console.log(e);
                }

                return this.ready;
            }

            get httpRequest() {
                return httpDefer.promise;
            }

            set data(dt) {
                this._data = dt;
                this.offlineDataAvaiable = true;
                httpDefer.resolve(dt);
            }

            get data() {
                return this._data;
            }

            get ls_keyName() {
                return ('idb_config_' + this.dbName + '_' + this.storeName);
            }

            get _localParams() {
                return $localStorage.getItem(this.ls_keyName);
            }

            _done({ data, getHttpData, lastUpdate }) {

                this.data = data;


                const ls_store_parameters = this._localParams;


                const cfg = "undefined" !== typeof ls_store_parameters && ls_store_parameters ? ls_store_parameters : {};

                if (!cfg.dateOfCreation)
                    cfg.dateOfCreation = this.toDay();

                const getTime = () => {
                    try {
                        return Date.hasOwnProperty('now') ? Date.now() : new Date().getTime();
                    } catch (e) {
                        return new Date().getTime();
                    }
                }


                if (lastUpdate || !cfg.lastDataUpdate) {
                    cfg.lastDataUpdate = { date: this.toDay() };
                    cfg.lastDataUpdate.time = getTime();
                }

                cfg.lastUpdate = { date: this.toDay(), time: getTime() };


                $localStorage.setItem(this.ls_keyName, cfg);


                this._resolve(getHttpData);

            }

            _getStoreData(dt) {
                const defer = $q.defer();

                this.$http({
                    method: 'POST',
                    url: '/' + this.dbName.toLowerCase() + '/' + this.storeName + '/',
                    cache: true,
                    data: dt,
                    transformResponse: (data, headersGetter, status) => {
                        if ("string" === typeof data)
                            return JSON.parse(data);

                        if (Array.isArray(data) && Array.isArray(data[0]))
                            return data[0];

                        return data;
                    }
                }).then(defer.resolve, defer.reject).catch(e => angular.noop);

                return defer.promise;
            }



            _updateLocalData({ remoteData, localData }) {

                if (!remoteData || !remoteData.length)
                    return this._done({ data: localData, getHttpData: 1, lastUpdate: 1 });

                this.objectStore.bulkUpdate(remoteData,
                    () => {
                        console.log('ok update ' + this.storeName);
                        this._getAll()
                            .then(
                                updateData => {
                                    this._done({ data: updateData, getHttpData: 1, lastUpdate: 1 });
                                },
                                err => {
                                    this._done({ data: localData, getHttpData: 1, lastUpdate: 1 });
                                }
                            )
                            .catch(err => {
                                this._done({ data: localData, getHttpData: 1, lastUpdate: 1 });
                            })
                            .done();
                    },

                    err => {
                        this._done({ data: localData, getHttpData: 1, lastUpdate: 0 });
                    },

                    n => this._notify({
                        'index': n && n.hasOwnProperty('index') ? n.index : null,
                        'getHttpData': true,
                        'dataLength': remoteData && Array.isArray(remoteData) ? remoteData.length : null
                    })

                );


            }

            _getAll() {
                return this.objectStore.getAll();
            }



        }

        return new IDBStore($http, toDay, db, dbName, storeName);
    };
};
IDBStore.$inject = ['$http', 'toDay'];
angular_module.service('IDBStore_setter', IDBStore);